package com.rp.util.sort;

import com.rp.util.sort.ISort;
import com.rp.util.sort.SortException;
import com.rp.util.ReflectionUtil;
import java.util.Comparator;
import java.util.List;

public final class InsertionSort implements ISort
{
	
	public InsertionSort ()
	{}

	public void sort (int[] arr)
	{
		sort (arr, 0, arr.length-1);
	}
	
	public void sort (int[] arr, int begin, int end)
	{
		int key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}	
			arr[i+1] = key;
		}
	}
	
	public void sort (String[] arr)
	{
		sort (arr, 0, arr.length-1);
	}
	
	public void sort (String[] arr, int begin, int end)
	{
		String key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && (arr[i].compareToIgnoreCase(key)) > 0)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (long[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}
	
	public void sort (long[] arr, int begin, int end)
	{
		long key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}

	public void sort (short[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}

	public void sort (short[] arr, int begin, int end)
	{
		short key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (double[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}

	public void sort (double[] arr, int begin, int end)
	{
		double key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (float[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}

	public void sort (float[] arr, int begin, int end)
	{
		float key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (char[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}

	public void sort (char[] arr, int begin, int end)
	{
		char key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (byte[] arr)
	{
		sort (arr, 0, arr.length-1);	
	}
	
	public void sort (byte[] arr, int begin, int end)
	{
		byte key;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			int i = index - 1;
			while (i >= begin && arr[i] > key)
			{
				arr[i+1] = arr[i];
				i = i - 1;	
			}
			arr[i+1] = key;
		}	
	}
	
	public void sort (Object[] arr, Comparator c, String methodName) throws SortException
	{
		sort (arr, c, methodName, new Class[0], new Object[0]);	
	}
	
	public void sort (Object[] arr, Comparator c, String methodName, Class[] paramTypes, Object[] paramValues)  
							throws SortException
	{
		sort (arr, 0, arr.length-1, c, methodName, paramTypes, paramValues);	
	}
	
	public void sort (Object[] arr, int begin, int end, Comparator c, String methodName)  
							throws SortException
	{
		sort (arr, begin, end, c, methodName, new Class[0], new Object[0]);
	}
	
	public void sort (Object[] arr, int begin, int end, Comparator c, String methodName, 
							Class[] paramTypes, Object[] paramValues)  
							throws SortException
	{
		try
		{
			Object key;
			for (int index = begin+1; index <= end; index++)
			{
				key = arr[index];
				int i = index - 1;
				while (i >= begin && (c.compare 
						(ReflectionUtil.getMethodValue (arr[i], methodName, paramTypes, paramValues), 
						ReflectionUtil.getMethodValue (key, methodName, paramTypes, paramValues))) > 0 )
				{
					arr[i+1] = arr[i];
					i = i - 1;	
				}
				arr[i+1] = key;
			}
		}
		catch (Exception e)
		{
			throw new SortException (e);	
		}
	}
	
	public void sort (Object[] arr, Comparator c)
	{
		sort (arr, 0, arr.length-1, c);	
	}
	
	public void sort (Object[] arr, int begin, int end, Comparator c)
	{
		Object key;
		int i;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr[index];
			i = index - 1;
			while (i >= begin && ( c.compare(arr[i], key)) > 0)
			{
				arr[i+1] = arr[i];
				i = i - 1;
			}
			arr[i+1] = key;	
		}	
	}
	
	public void sort (List arr, Comparator c)
	{
		sort (arr, 0, arr.size()-1, c);	
	}
	
	public void sort (List arr, int begin, int end, Comparator c)
	{
		Object key;
		int i;
		for (int index = begin+1; index <= end; index++)
		{
			key = arr.get(index);
			i = index - 1;
			while (i >= begin && ( c.compare(arr.get(i), key)) > 0)
			{
				arr.set(i+1, arr.get(i));
				i = i - 1;
			}
			arr.set(i+1, key);	
		}	
	}
}